#!/usr/bin/perl

use strict;
use warnings;
use Audio::Wav;
use CGI;
use Template;

my $root = ".";

# Directory containing input wave files (relative)
my $person = "shitting";

# Directory to create output wave files (relative)
my $wavdir = "waves";


my $q = new CGI;

my $sentence = $q->param("sentence");

if ($sentence)
{
	my $filename = make_sentence_speech($sentence, $person);
	print "Location: $wavdir/$filename\n";
	print "Content-type: audio/wav\n\n";
}
else 
{
	my @words = wordfiles_for_person($person);
	output("index", { words => \@words });
}

sub	make_sentence_speech
{
	my ($sentence, $person) = @_;
	my @wordfiles = wordfiles_for_sentence($sentence, $person);
	my $wav = new Audio::Wav;

	my $details = {
		bits_sample	=> 16,
		sample_rate	=> 44100,
		channels	=> 2,
		no_cache	=> 1,
	};

	my $filename = $person . "." . time() . ".wav";
	
	my $write = $wav->write("$wavdir/$filename", $details);

	foreach my $file (@wordfiles)
	{
		my $read = $wav->read("$root/$person/$file.wav");
		$write->write_raw_samples($read->read_raw_samples($read->length_samples));
		
		$write->write_raw(pack("H" x 10000, 0))
	}

	$write->finish;

	return $filename;
}

sub	wordfiles_for_person
{
	my ($person) = @_;

	opendir (local *D, "$root/$person")
		or error("Couldn't open dir");
	my @wordfiles = sort map { s/\..*//; $_ } grep { /\w/ } readdir D;
	closedir D;

	return @wordfiles;
}

sub	wordfiles_for_sentence	
{
	my ($sentence, $person) = @_;

	my @sentence_words = grep { /\S/ } split(/ /, $sentence);
	my @sentence_wordfiles;

	my @wordfiles = wordfiles_for_person($person);

	while (@sentence_words)
	{
		my @subsentences = map
			{ join("_", @sentence_words[0..$_]) }
		0..$#sentence_words;
		my $n = $#subsentences;
		my $match;
		while ($n >= 0)
		{
			if (grep { $_ eq $subsentences[$n] } @wordfiles)
			{
				$match = $subsentences[$n];
				push(@sentence_wordfiles, $match);
				splice(@sentence_words, 0, $n + 1);
				last;
			}

			$n--;
		}

		if (!defined $match)
		{
			error("No match for word <b>$sentence_words[0]</b>.<br />Please only use listed words.");
		}
	}

	return @sentence_wordfiles;
}

sub	output
{
	my ($template, $vars) = @_;
	my $t = Template->new(
		INCLUDE_PATH	=> "templates"
	);
	my $output;
	$t->process("$template.tt2", $vars, \$output)
		or die $t->error;

	print "Content-type: text/html\n\n";
	print $output;
}

sub	error {
	my ($errstr) = @_;
	output("error", { error => $errstr });
	exit 1;
}
